@@ -1,9 +1,20 @@
 from __future__ import print_function, absolute_import, division
 
+import contextlib
+import warnings
+try:
+    import builtins
+except ImportError:
+    # python2
+    import __builtin__ as builtins
+
 import numpy as np
 from astropy.wcs import (WCSSUB_SPECTRAL, WCSSUB_LONGITUDE, WCSSUB_LATITUDE)
 from . import wcs_utils
 from astropy import log
+from astropy.io.fits import BinTableHDU, Column
+from astropy import units as u
+import itertools
 
 def _fix_spectral(wcs):
     """
@@ -22,7 +33,7 @@ def _fix_spectral(wcs):
 
     # sanitize noncompliant headers
     if 'spectral' not in types:
-        log.warn("No spectral axis found; header may be non-compliant.")
+        log.warning("No spectral axis found; header may be non-compliant.")
         for ind,tp in enumerate(types):
             if tp not in ('celestial','stokes'):
                 if wcs.wcs.ctype[ind] in wcs_utils.bad_spectypes_mapping:
@@ -67,7 +78,7 @@ def _split_stokes(array, wcs):
         if types.count('celestial') == 2 and types.count('spectral') == 1:
             if None in types:
                 stokes_index = types.index(None)
-                log.warn("FITS file has no STOKES axis, but it has a blank"
+                log.warning("FITS file has no STOKES axis, but it has a blank"
                          " axis type at index {0} that is assumed to be "
                          "stokes.".format(4-stokes_index))
             else:
@@ -76,7 +87,7 @@ def _split_stokes(array, wcs):
                         stokes_index = ii
                         stokes_type = tp
 
-                log.warn("FITS file has no STOKES axis, but it has an axis"
+                log.warning("FITS file has no STOKES axis, but it has an axis"
                          " of type {1} at index {0} that is assumed to be "
                          "stokes.".format(4-stokes_index, stokes_type))
         else:
@@ -126,7 +137,7 @@ def _orient(array, wcs):
     if wcs.wcs.naxis != 3:
         raise ValueError("Input WCS must be 3-dimensional")
 
-    wcs = _fix_spectral(wcs)
+    wcs = wcs_utils.diagonal_wcs_to_cdelt(_fix_spectral(wcs))
 
     # reverse from wcs -> numpy convention
     axtypes = wcs.get_axis_types()[::-1]
@@ -230,3 +241,71 @@ def iterator_strategy(cube, axis=None):
     if cube.size < 1e8:  # smallish
         return 'cube'
     return 'slice'
+
+
+def beams_to_bintable(beams):
+    """
+    Convert a list of beams to a CASA-style BinTableHDU
+    """
+
+    c1 = Column(name='BMAJ', format='1E', array=[bm.major.to(u.arcsec).value for bm in beams], unit=u.arcsec.to_string('FITS'))
+    c2 = Column(name='BMIN', format='1E', array=[bm.minor.to(u.arcsec).value for bm in beams], unit=u.arcsec.to_string('FITS'))
+    c3 = Column(name='BPA', format='1E', array=[bm.pa.to(u.deg).value for bm in beams], unit=u.deg.to_string('FITS'))
+    c4 = Column(name='CHAN', format='1J', array=[bm.meta['CHAN'] if 'CHAN' in bm.meta else 0 for bm in beams])
+    c5 = Column(name='POL', format='1J', array=[bm.meta['POL'] if 'POL' in bm.meta else 0 for bm in beams])
+
+    bmhdu = BinTableHDU.from_columns([c1, c2, c3, c4, c5])
+    bmhdu.header['EXTNAME'] = 'BEAMS'
+    bmhdu.header['EXTVER'] = 1
+    bmhdu.header['XTENSION'] = 'BINTABLE'
+    bmhdu.header['NCHAN'] = len(beams)
+    bmhdu.header['NPOL'] = len(set([bm.meta['POL'] for bm in beams]))
+    return bmhdu
+
+def average_beams(beams, includemask=None):
+    """
+    Average the beam major, minor, and PA attributes.
+
+    This is usually a dumb thing to do!
+    """
+
+    from radio_beam import Beam
+    from astropy.stats import circmean
+
+    if includemask is None:
+        includemask = itertools.cycle([True])
+
+    major = u.Quantity([bm.major for bm,incl in zip(beams,includemask) if incl], u.deg)
+    minor = u.Quantity([bm.minor for bm,incl in zip(beams,includemask) if incl], u.deg)
+    pa = u.Quantity([bm.pa for bm,incl in zip(beams,includemask) if incl], u.deg)
+    new_beam = Beam(major=major.mean(), minor=minor.mean(),
+                    pa=circmean(pa, weights=major/minor))
+
+    return new_beam
+
+@contextlib.contextmanager
+def _map_context(numcores):
+    """
+    Mapping context manager to allow parallel mapping or regular mapping
+    depending on the number of cores specified.
+    """
+    if numcores is not None and numcores > 1:
+        try:
+            import multiprocessing
+            p = multiprocessing.Pool(processes=numcores)
+            map = p.map
+            parallel = True
+        except ImportError:
+            map = builtins.map
+            warnings.warn("Could not import multiprocessing.  "
+                          "map will be non-parallel.")
+            parallel = False
+    else:
+        parallel = False
+        map = builtins.map
+
+    try:
+        yield map
+    finally:
+        if parallel:
+            p.close()
