@@ -256,6 +256,30 @@ def test_wcs_validity_check():
     # just checking that this works, not that it does anything in particular
     moment_map = s3.moment(order=1)
 
+def test_wcs_validity_check_failure():
+    cube, data = cube_and_raw('adv.fits')
+    wcs2 = cube.wcs.copy()
+    # add some difference in the 4th decimal place
+    wcs2.wcs.crval[2] += 0.00001
+
+    # can make a mask
+    mask = BooleanArrayMask(data>0, cube._wcs)
+
+    # but if it's not exactly equal, an error should be raised at this step
+    with pytest.raises(ValueError) as exc:
+        cube = cube.with_mask(mask)
+    assert exc.value.args[0] == "WCS does not match mask WCS"
+
+    # this one should work though
+    cube = cube.with_mask(mask, wcs_tolerance=1e-4)
+    assert cube._wcs_tolerance == 1e-4
+
+    # then the rest of this should be OK
+    s2 = cube.spectral_slab(-2 * u.km / u.s, 2 * u.km / u.s)
+    s3 = s2.with_spectral_unit(u.km / u.s, velocity_convention=u.doppler_radio)
+    # just checking that this works, not that it does anything in particular
+    moment_map = s3.moment(order=1)
+
 def test_mask_spectral_unit_functions():
     cube, data = cube_and_raw('adv.fits')
 
@@ -373,6 +397,8 @@ def test_include():
 
 def test_1d_mask():
     # regression test for issue revealed in #183
+    # In principle, this is also a regression test for #298, except this always
+    # passed where #298 failed, which I don't understand.
 
     cube, data = cube_and_raw('adv.fits')
     mask = np.array([True, False, True, False])
@@ -382,3 +408,15 @@ def test_1d_mask():
 
     np.testing.assert_almost_equal(sum0.value, sum0d)
 
+def test_1d_mask_amp():
+    cube, data = cube_and_raw('adv.fits')
+    mask = np.array([True, False, True, False])
+
+    Mask = BooleanArrayMask(mask[:,None,None],
+                            wcs=cube._wcs,
+                            shape=cube.shape,
+                           )
+
+    ampd = cube.mask & Mask
+
+    ampd.include()
