@@ -175,7 +175,7 @@ def axis_names(wcs):
     return names
 
 
-def slice_wcs(mywcs, view, numpy_order=True):
+def slice_wcs(mywcs, view, shape=None, numpy_order=True):
     """
     Slice a WCS instance using a Numpy slice. The order of the slice should
     be reversed (as for the data) compared to the natural WCS order.
@@ -216,11 +216,31 @@ def slice_wcs(mywcs, view, numpy_order=True):
             # None,None,step or None,stop,step)
             iview = slice(0, iview.stop, iview.step)
 
-        if iview.start is not None:
-            if numpy_order:
-                wcs_index = mywcs.wcs.naxis - 1 - i
+        if numpy_order:
+            wcs_index = mywcs.wcs.naxis - 1 - i
+        else:
+            wcs_index = i
+
+        if iview.step is not None and iview.step < 0:
+            if iview.step != -1:
+                raise NotImplementedError("Haven't dealt with resampling & reversing.")
+            # reverse indexing requires the use of shape
+            if shape is None:
+                raise ValueError("Cannot reverse-index a WCS without "
+                                 "specifying a shape.")
+            if iview.stop is not None:
+                refpix = iview.stop
             else:
-                wcs_index = i
+                refpix = shape[i]
+            # this will raise an inconsistent axis type error if slicing over
+            # celestial axes is attempted
+            # wcs_index+1 is required because sub([0]) = sub([all])
+            crval = mywcs.sub([wcs_index+1]).wcs_pix2world([refpix], 0)[0]
+            crpix = 1
+            wcs_new.wcs.crpix[wcs_index] = crpix
+            wcs_new.wcs.crval[wcs_index] = crval
+
+        elif iview.start is not None:
 
             if iview.step not in (None, 1):
                 crpix = mywcs.wcs.crpix[wcs_index]
@@ -228,16 +248,21 @@ def slice_wcs(mywcs, view, numpy_order=True):
                 # equivalently (keep this comment so you can compare eqns):
                 # wcs_new.wcs.crpix[wcs_index] =
                 # (crpix - iview.start)*iview.step + 0.5 - iview.step/2.
-                crp = ((crpix - iview.start - 1.)/iview.step
-                       + 0.5 + 1./iview.step/2.)
+                crp = ((crpix - iview.start - 1.)/iview.step +
+                       0.5 + 1./iview.step/2.)
                 wcs_new.wcs.crpix[wcs_index] = crp
                 wcs_new.wcs.cdelt[wcs_index] = cdelt * iview.step
             else:
                 wcs_new.wcs.crpix[wcs_index] -= iview.start
+
+    # Without this, may cause a regression of #234
+    wcs_new.wcs.set()
+
     return wcs_new
 
-def check_equality(wcs1, wcs2, warn_missing=False, ignore_keywords=['MJD-OBS',
-                                                                    'VELOSYS']):
+def check_equality(wcs1, wcs2, warn_missing=False,
+                   ignore_keywords=['MJD-OBS', 'VELOSYS'],
+                   wcs_tolerance=0.0):
     """
     Check if two WCSs are equal
 
@@ -250,7 +275,24 @@ def check_equality(wcs1, wcs2, warn_missing=False, ignore_keywords=['MJD-OBS',
     ignore_keywords: list of str
         Keywords that are stored as part of the WCS but do not define part of
         the coordinate system and therefore can be safely ignored.
+    wcs_tolerance : float
+        The decimal level to check for equality.
+        For example, 1e-2 would have 0.001 and 0.002 as equal, but 1e-3 would
+        have them as inequal
     """
+    # TODO: use this to replace the rest of the check_equality code
+    #return wcs1.wcs.compare(wcs2.wcs, cmp=wcs.WCSCOMPARE_ANCILLARY,
+    #                        tolerance=tolerance)
+    #Until we've switched to the wcs.compare approach, we need to have
+    #np.testing.assert_almost_equal work
+    if wcs_tolerance == 0:
+        exact = True
+    else:
+        exact = False
+        # np.testing.assert_almost_equal wants an integer
+        # e.g., for 0.0001, the integer is 4
+        decimal = int(np.ceil(-np.log10(wcs_tolerance)))
+
 
     # naive version:
     # return str(wcs1.to_header()) != str(wcs2.to_header())
@@ -281,7 +323,10 @@ def check_equality(wcs1, wcs2, warn_missing=False, ignore_keywords=['MJD-OBS',
                         log.debug("Header 1, {0}: {1} != {2}".format(key,u1,u2))
             elif isinstance(c1[1], (float, np.float)):
                 try:
-                    np.testing.assert_almost_equal(c1[1], c2[1])
+                    if exact:
+                        assert c1[1] == c2[1]
+                    else:
+                        np.testing.assert_almost_equal(c1[1], c2[1], decimal=decimal)
                 except AssertionError:
                     if key in ('RESTFRQ','RESTWAV'):
                         warnings.warn("{0} is not equal in WCS; ignoring ".format(key)+
@@ -334,8 +379,8 @@ def strip_wcs_from_header(header):
         if kw not in keys_to_keep:
             del newheader[kw]
 
-    for kw in ('CRPIX{ii}', 'CRVAL{ii}', 'CDELT{ii}', 'CUNIT{ii}',
-               'CTYPE{ii}', 'PC0{ii}_0{jj}', 'CD{ii}_{jj}',):
+    for kw in ('CRPIX{ii}', 'CRVAL{ii}', 'CDELT{ii}', 'CUNIT{ii}', 'CTYPE{ii}',
+               'PC0{ii}_0{jj}', 'CD{ii}_{jj}', 'CROTA{ii}'):
         for ii in range(5):
             for jj in range(5):
                 k = kw.format(ii=ii,jj=jj)
@@ -344,3 +389,17 @@ def strip_wcs_from_header(header):
 
 
     return newheader
+
+def diagonal_wcs_to_cdelt(mywcs):
+    """
+    If a WCS has only diagonal pixel scale matrix elements (which are composed
+    from cdelt*pc), use them to reform the wcs as a CDELT-style wcs with no pc
+    or cd elements
+    """
+    offdiag = ~np.eye(mywcs.pixel_scale_matrix.shape[0], dtype='bool')
+    if not any(mywcs.pixel_scale_matrix[offdiag]):
+        cdelt = mywcs.pixel_scale_matrix.diagonal()
+        del mywcs.wcs.pc
+        del mywcs.wcs.cd
+        mywcs.wcs.cdelt = cdelt
+    return mywcs
